/* Include all header files used by dialog definitions. */
#include "dlginc.h"

/*****************************************************************************
   Because these functions are called by ColorWorks through pointers, the
   calling convention must be "system" rathar than "optlink".  The "system"
   calling convention passes a functions parameters through the stack, which
   is the normal method for a "C" function.  The "optlink" convention
   passes parameters to a function through the registers.  This Example assumes
   you are using the IBM VisualAge C++ compiler, which is what this example was
   developed with.
******************************************************************************/
#pragma linkage (InitilizePlugin,                  system)
#pragma linkage (PluginEntryPoint,                 system)

/* General defines used by the plugin example. */
#define TASK_COMPLETED                     -1
#define PLUGIN_EFFECT                      26
#define UNDO_TRANSFORM                      0
#define NO                              FALSE
#define YES                              TRUE
#define CHECKED                             1
#define MOVE_RIGHT                          1
#define MOVE_LEFT                          -1
#define MOVE_UP                            -1
#define MOVE_DOWN                           1
#define NO_MOVEMENT                         0
#define MOVEMENT_IN_BOTH_DIRECTIONS         1
#define ERROR                           NO_ERROR-1
#define BYTES_PER_PIXEL                     3
#define PLUGINEFFECT                        0

/* Macro definitions. */
/* This define checks the value of a parameter to make sure it is between
   0 and 255.  If the value is less than 0 it is set to 0, if the value is
   greater than 255 it is set to 255. */
#define CHECKPIXELBOUNDS(x) if(x<0) x=0; else if(x>255) x=255;

/* Stringtable definitions. */
#define STRING1  1
#define STRING2  2

/*****************************************************************************
   This structure is passed to the plugins initilization funtion, "InitilizePlugin()".
   The plugins initilization function is called once, when ColorWorks is first
   started.   This structure is filled in by the plugin with information
   needed by ColorWorks in order to execute the plugin.
******************************************************************************/
typedef struct _PluginInitilization
        {
/* Data allocated by the plugins initilization.  This data is available to the
   plugin for the entire ColorWorks session. */
           VOID  *StaticData;
/* Reserved dont change contents of reserved variables. */
           CHAR  Reserved1[256];
/* This is the name of the function ColorWorks should call when the user
   selects your plugin from the ColorWorks menu. */
           CHAR  EffectEntryPoint[100];
/* This is the name of your effect.  This name will appear in the ColorWorks
   menu, allowing the user to choose your effect by this name */
           CHAR  EffectName[100];
/* This is the text which will appear in the  "Undo/Redo Float" menu item of
   the "Edit" menu. */
           CHAR  UndoName[100];
/* This is the handle to ColorWorks' anchor block. */
           HAB   HwndAB;
/* This is the module handle to the plugin DLL. */
           HMODULE ModuleHandle;
/* This is the catagory of the plugin.  ColorWorks supports several types of
   plugins, each catagorie of plugin performs a different main task and is
   placed in a different menu in the ColorWorks menu stucture. */
           CHAR    PluginCatagory;
        };

/*****************************************************************************
   A floated image can be any shape, and floats which are not rectangular
   have areas which are transparant.  These transparant portions of the float
   have a predefined pixel value that ColorWorks recognizes as transparant,
   and handles as such when manipulating the float.  This structure contains
   the values of the color components which comprise a transparant pixel.
   See function "IsPixelTransparant" for additional details / example of use.
******************************************************************************/
typedef struct _TransparentColor
        {
           UCHAR TransparentRed;
           UCHAR TransparentGreen;
           UCHAR TransparentBlue;
        };

/*****************************************************************************
   This structure contains information which is needed to access the floated
   image data.  The structure member "ImageDataPointer" is the pointer to
   the floated image.  The pointer points to the lower left corner of the
   float rectangle.
******************************************************************************/
typedef struct _FloatData
        {
/* This is the pointer to the floated image data. This points to the lower left
   corner of the float rectangle. */
           UCHAR *ImageDataPointer;
/* This is the width if the floated image data in pixels. */
           INT   Width;
/* This is the height if the floated image data in pixels. */
           INT   Height;
/* This is the the size of a row of floated image data in bytes. */
           INT   RowsizeInBytes;
/* These next two variables are the current location of the float on the image.
   This location coorosponds the lower left corner of the floated. */
           INT   MergePointX;
           INT   MergePointY;
        };

/*****************************************************************************
   This structure contains information which is needed to access the
   image data.
******************************************************************************/
typedef struct _ImageData
        {
/* This is the width if the image data in pixels. */
           INT   Width;
/* This is the height if the image data in pixels. */
           INT   Height;
/* This is the the size of a row of image data in bytes. */
           INT   RowsizeInBytes;
        };

/*****************************************************************************
   This structure contains the pointers to ColorWorks callback functions.
   These functions perform services which your plugin might want or need to use.
******************************************************************************/
typedef struct _CallbackFunctions
        {
/* This is a pointer to the ColorWorks function which may be used to center any
   dialog boxes your plugin may use. */
           VOID *CenterDialogBox;
/* This is a pointer to the ColorWorks function which is used to set the canvas
   windows completion ribbon. */
           VOID *SetCompletionRibbon;
/* This is a pointer to the ColorWorks function which is used to merge the float
   with the canvas. After the effect is applied this function must be called in
   order to make the results of the effect visible. */
           VOID *MergeFloatWithVideobuffer;
/* This is a pointer to the ColorWorks function which is used to save an undo
   of the image data. If the user chooses to execute your plugin this function
   should be called, otherwise the user will not be able to undo your plugins effect. */
           VOID *SaveTransformUndo;
/* This is a pointer to the ColorWorks function which will get a row of image
   data from the actual image, not the float.  The the only way your plugin
   can get data from the actual image is to use this function or the function
   "GetPixelFromImage".  Direct access to the image buffer is not provided
   because the image may be compressed, and getting to compressed data is
   MESSY, so to keep it simple, ColorWorks gets the data for you. */
           VOID *GetRowFromImage;
/* This is a pointer to the ColorWorks function which will get a pixel
   from the actual image, not the float.  See comment above. */
           VOID *GetPixelFromImage;
/* If a plugin effect creates a new float, as in the case of a plugin which
   changes the dimentions of the float, this function must be called before
   merging the float with the videobuffer.  This function performs two
   tasks, first it informs ColorWorks of the dimentions of the new float,
   and second, it provides colorWorks with the address of the new float data. */
           VOID *NewFloatWasCreated;
/* This is a pointer to the ColorWorks callback function which creates, for the
   plugin, a preview window.  If your plugin uses a dialog, and in that dialog
   you wish to have a preview, this function, along with the next can be used
   to aleviate much of the work needed to create a preview. */
           VOID *CreatePreviewWindow;
/* This is a pointer to the ColorWorks callback function which displays the
   floated image in the preview window. */
           VOID *DisplayPreview;
        };

/*****************************************************************************
   This structure is passed to your plugins entry point procedure, in this
   exmple that procedure is "PluginEntryPoint()".  This structure contains
   information needed by your plugin in order to perform its effect.
*****************************************************************************/
typedef struct _PluginRuntime
        {
/* This structure contains the values of the color components which comprise
   a transparant pixel.  See structure definition for additional details. */
           struct  _TransparentColor TransparentColor;
/* This structure contains information which is needed to access the floated
   image data. For example the floats width, height, etc. */
           struct _FloatData FloatData;
/* This structure describes the dimentions of the image the user is currently
   working on. */
           struct _ImageData ImageData;
/* This structure contains the pointers to callback functions. These functions
   perform services your plugin might want or need to use.*/
           struct _CallbackFunctions CallbackFunctions;
/* This is the data area allocated by your plugins initilization function. */
           VOID  *StaticData;
/* This the number of CPUs in the system and available for ColorWorks' use. */
           CHAR  CPUCount;
/* This is the handle to ColorWorks' client window. */
           HWND  HwndCWClientWindow;
/* This is the handle to ColorWorks' frame window. */
           HWND  HwndCWFrameWindow;
/* This is the module handle to the plugin DLL. */
           HMODULE ModuleHandle;
/* This is the handle to the canvas, this variable is needed by several of the
   ColorWorks callback functions. */
           VOID *HwndCanvas;
/* This is the catagory id assigend to the plugin DLL by ColorWorks.*/
           CHAR PluginCatagory;
/* This is the handle to ColorWorks' anchor block. */
           HAB  HwndAB;
        };

/*****************************************************************************
   This structure contains the settings for the motion blurs dialog controls.
   This structure is allocated by the "InitilizePlugin" function, the pointer
   to this data area is placed in the "StaticData" member of the
   "PluginInitilization structure.  See code comments  inside function
   "InitilizePlugin" for more details.
******************************************************************************/
typedef struct _MotionBlurDialogSettings
        {
/* This variable contains the horizontal movement.
   1=move right, 0=no movement, -1=move left. */
           INT  HorizontalDirection;
/* This variable contains the vertical movement.
   1=move down, 0=no movement, -1=move up. */
           INT  VerticalDirection;
/* This variable contains the velocity of the motion blur.  This number equals
   the number of pixels the image is to be blured in the selected direction(s). */
           CHAR BlurVelocity;
/* This the handle to the frame window of the moation blurs dialog preview window. */
           HWND HwndPreviewWindowFrame;
/* This the handle to the client window of the moation blurs dialog preview window. */
           HWND HwndPreviewWindowClient;
        };

/* Function prototypes */
INT  InitilizePlugin(struct _PluginInitilization *PluginInitilization);
VOID PluginEntryPoint(struct _PluginRuntime *PluginRuntime);
VOID LoadMotionBlurDialog(struct _PluginRuntime *RuntimeStructure);
MRESULT EXPENTRY MotionBlurDialogProcedure(HWND   HwndDialog, ULONG  message,
                                           MPARAM param1,     MPARAM param2);
VOID InitilizeMotionBlurDialog(HWND   HwndDialog, ULONG  message,
                               MPARAM param1,     MPARAM param2);
VOID ApplyMotionBlur(struct _PluginRuntime *PluginRuntime, CHAR BlurVelocity, INT hor, INT vert);
LONG QueryNumericSpinButton(HWND HwndDialog, ULONG SpinButtonID);
BOOL IsPixelTransparant(UCHAR *PixelPointer, struct _PluginRuntime *PluginRuntime);
VOID LoadSystemPointer(LONG PointerID);
VOID ScrubColor(INT *Blue, INT *Green, INT *Red, struct _PluginRuntime *PluginRuntime);
VOID GetMotionBlurDirection(HWND HwndDialog, INT *HorizontalDirection,
                            INT *VerticalDirection);
USHORT IsRadioButtonChecked(HWND HwndDialog, ULONG RadioButtonID);
VOID ExampleFunction1(struct _PluginRuntime *PluginRuntime);
UCHAR *CreateANewFloat(struct _PluginRuntime *PluginRuntime,
                       INT FloatWidth, INT FloatHeight);
MRESULT EXPENTRY PreviewWindowProcedure(HWND HwndClient, ULONG message, MPARAM param1, MPARAM param2);
VOID ApplyMotionBlurToPreview(HWND HwndClient, UCHAR *RowDataPointer,
                              INT Y, INT PointsInDataPointerArray);
VOID MotionBlurThePreview(HWND HwndClient, UCHAR *RowDataPointer, INT Y,
                          INT PointsInDataPointerArray, float ScaleX, float ScaleY,
                          struct _PluginRuntime *PluginRuntime, HWND HwndDialog);
VOID FreeMotionBlurDialogResources(struct _PluginRuntime *PluginRuntime);

